from .correios_webservice_client import CorreiosWSClient
from .sro import SRO
from .CEP import CEP
from .credenciais import Credenciais
from .cartao_postagem import CartaoPostagem
from .sigepweb import SigepWeb

